@@ -12,9 +12,6 @@
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
-#include <wx/file.h>
-#include <wx/filename.h>
-
 #ifndef __WXMSW__
 	#define mutex mutex_override // Sadly on some platforms system headers include conflicting names
 	#include <netdb.h>
@@ -368,10 +365,10 @@ int CControlSocket::CheckOverwriteFile()
 
 	CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
 
-	if (pData->download)
-	{
-		if (!wxFile::Exists(pData->localFile))
+	if (pData->download) {
+		if (fz::local_filesys::get_file_type(fz::to_native(pData->localFile), true) != fz::local_filesys::file) {
 			return FZ_REPLY_OK;
+		}
 	}
 
 	CDirentry entry;
@@ -436,15 +433,17 @@ CFileTransferOpData::~CFileTransferOpData()
 {
 }
 
-wxString CControlSocket::ConvToLocal(const char* buffer, size_t len)
+std::wstring CControlSocket::ConvToLocal(const char* buffer, size_t len)
 {
+	std::wstring ret;
+
 	size_t outLen{};
 	if (m_useUTF8) {
 		wxChar* out = ConvToLocalBuffer(buffer, wxConvUTF8, len, outLen);
 		if (out) {
-			wxString str(out, outLen - 1);
+			ret.assign(out, outLen - 1);
 			delete [] out;
-			return str;
+			return ret;
 		}
 
 		// Fall back to local charset on error
@@ -455,23 +454,24 @@ wxString CControlSocket::ConvToLocal(const char* buffer, size_t len)
 	}
 
 	if (m_pCSConv) {
-		wxChar* out = ConvToLocalBuffer(buffer, *m_pCSConv, len, outLen);
+		wchar_t* out = ConvToLocalBuffer(buffer, *m_pCSConv, len, outLen);
 		if (out) {
-			wxString str(out, outLen - 1);
+			ret.assign(out, outLen - 1);
 			delete [] out;
-			return str;
+			return ret;
 		}
 	}
 
 	wxCSConv conv(_T("ISO-8859-1"));
-	wxString str = conv.cMB2WX(buffer);
-	if (str.empty())
-		str = wxConvCurrent->cMB2WX(buffer);
+	ret = conv.cMB2WX(buffer);
+	if (ret.empty()) {
+		ret = wxConvCurrent->cMB2WX(buffer);
+	}
 
-	return str;
+	return ret;
 }
 
-wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen)
+wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen)
 {
 	wxASSERT(buffer && len > 0 && !buffer[len - 1]);
 	outlen = conv.ToWChar(0, 0, buffer, len);
@@ -483,7 +483,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, si
 	return unicode;
 }
 
-wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen)
+wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen)
 {
 	if (m_useUTF8) {
 #ifdef __WXMSW__
@@ -492,13 +492,13 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t
 		// This helps when processing large directory listings.
 		int outlen2 = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buffer, len, 0, 0);
 		if (outlen2 > 0) {
-			wxChar* out = new wxChar[outlen2];
+			wchar_t* out = new wchar_t[outlen2];
 			MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buffer, len, out, outlen2);
 			outlen = static_cast<size_t>(outlen2);
 			return out;
 		}
 #else
-		wxChar* res = ConvToLocalBuffer(buffer, wxConvUTF8, len, outlen);
+		wchar_t* res = ConvToLocalBuffer(buffer, wxConvUTF8, len, outlen);
 		if (res && *res)
 			return res;
 #endif
@@ -511,18 +511,18 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t
 	}
 
 	if (m_pCSConv) {
-		wxChar* res = ConvToLocalBuffer(buffer, *m_pCSConv, len, outlen);
+		wchar_t* res = ConvToLocalBuffer(buffer, *m_pCSConv, len, outlen);
 		if (res && *res)
 			return res;
 	}
 
 	// Fallback: Conversion using current locale
-	wxChar* res = ConvToLocalBuffer(buffer, *wxConvCurrent, len, outlen);
+	wchar_t* res = ConvToLocalBuffer(buffer, *wxConvCurrent, len, outlen);
 
 	return res;
 }
 
-wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /*=false*/)
+wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8)
 {
 	if (m_useUTF8 || force_utf8) {
 		wxCharBuffer const buffer = str.utf8_str();
@@ -1123,8 +1123,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 {
 	wxASSERT(pFileExistsNotification);
 
-	if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer)
-	{
+	if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
 		return false;
 	}
@@ -1207,22 +1206,33 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		break;
 	case CFileExistsNotification::rename:
 		if (pData->download) {
-			wxFileName fn = pData->localFile;
-			fn.SetFullName(pFileExistsNotification->newName);
-			pData->localFile = fn.GetFullPath();
+			{
+				wxString tmp;
+				CLocalPath l(pData->localFile, &tmp);
+				if (l.empty() || tmp.empty()) {
+					ResetOperation(FZ_REPLY_INTERNALERROR);
+					return false;
+				}
+				pData->localFile = l.GetPath() + tmp;
+			}
 
 			int64_t size;
 			bool isLink;
-			if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file)
+			if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file) {
 				pData->localFileSize = size;
-			else
+			}
+			else {
 				pData->localFileSize = -1;
+			}
 
-			if (CheckOverwriteFile() == FZ_REPLY_OK)
+			if (CheckOverwriteFile() == FZ_REPLY_OK) {
 				SendNextCommand();
+			}
 		}
 		else {
 			pData->remoteFile = pFileExistsNotification->newName;
+			pData->fileTime = fz::datetime();
+			pData->remoteFileSize = -1;
 
 			CDirentry entry;
 			bool dir_did_exist;
@@ -1231,15 +1241,13 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 				matched_case)
 			{
 				pData->remoteFileSize = entry.size;
-				if (entry.has_date())
+				if (entry.has_date()) {
 					pData->fileTime = entry.time;
+				}
 
-				if (CheckOverwriteFile() != FZ_REPLY_OK)
+				if (CheckOverwriteFile() != FZ_REPLY_OK) {
 					break;
-			}
-			else {
-				pData->fileTime = fz::datetime();
-				pData->remoteFileSize = -1;
+				}
 			}
 
 			SendNextCommand();
@@ -1273,7 +1281,7 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 	if (local_path.empty() || !local_path.HasParent())
 		return;
 
-	// Only go back as far as needed. On comparison, wxWidgets'
+	// Only go back as far as needed. By comparison, wxWidgets'
 	// wxFileName::Mkdir always starts at the root.
 	std::vector<wxString> segments;
 	while (!local_path.Exists() && local_path.HasParent()) {
