@@ -67,14 +67,59 @@ public:
 };
 
 namespace {
-#if GNUTLS_VERSION_NUMBER >= 0x030400
 extern "C" int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming, gnutls_datum_t const*)
-#else
-extern "C" int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming)
-#endif
 {
 	return CTlsSocketCallbacks::handshake_hook_func(session, htype, post, incoming);
 }
+
+struct cert_list_holder final
+{
+	cert_list_holder() = default;
+	~cert_list_holder() {
+		for (unsigned int i = 0; i < certs_size; ++i) {
+			gnutls_x509_crt_deinit(certs[i]);
+		}
+		gnutls_free(certs);
+	}
+
+	cert_list_holder(cert_list_holder const&) = delete;
+	cert_list_holder& operator=(cert_list_holder const&) = delete;
+
+	gnutls_x509_crt_t * certs{};
+	unsigned int certs_size{};
+};
+
+struct datum_holder final : gnutls_datum_t
+{
+	datum_holder() {
+		data = 0;
+	}
+
+	~datum_holder() {
+		gnutls_free(data);
+	}
+
+	datum_holder(datum_holder const&) = delete;
+	datum_holder& operator=(datum_holder const&) = delete;
+
+};
+
+void clone_cert(gnutls_x509_crt_t in, gnutls_x509_crt_t &out)
+{
+	gnutls_x509_crt_deinit(out);
+	out = 0;
+
+	if (in) {
+		datum_holder der;
+		if (gnutls_x509_crt_export2(in, GNUTLS_X509_FMT_DER, &der) == GNUTLS_E_SUCCESS) {
+			gnutls_x509_crt_init(&out);
+			if (gnutls_x509_crt_import(out, &der, GNUTLS_X509_FMT_DER) != GNUTLS_E_SUCCESS) {
+				gnutls_x509_crt_deinit(out);
+				out = 0;
+			}
+		}
+	}
+}
 }
 
 CTlsSocket::CTlsSocket(event_handler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
@@ -122,6 +167,9 @@ bool CTlsSocket::Init()
 		return false;
 	}
 
+	// Disable time checks. We allow expired/not yet valid certificates, though only after explicit user confirmation
+	gnutls_certificate_set_verify_flags(m_certCredentials, gnutls_certificate_get_verify_flags(m_certCredentials) | GNUTLS_VERIFY_DISABLE_TIME_CHECKS | GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS);
+
 	if (!InitSession())
 		return false;
 
@@ -477,7 +525,7 @@ void CTlsSocket::OnSend()
 
 bool CTlsSocket::CopySessionData(const CTlsSocket* pPrimarySocket)
 {
-	gnutls_datum_t d;
+	datum_holder d;
 	int res = gnutls_session_get_data2(pPrimarySocket->m_session, &d);
 	if (res) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_session_get_data2 on primary socket failed: %d"), res);
@@ -486,15 +534,16 @@ bool CTlsSocket::CopySessionData(const CTlsSocket* pPrimarySocket)
 
 	// Set session data
 	res = gnutls_session_set_data(m_session, d.data, d.size );
-	gnutls_free(d.data);
 	if (res) {
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("gnutls_session_set_data failed: %d. Going to reinitialize session."), res);
 		UninitSession();
-		if (!InitSession())
+		if (!InitSession()) {
 			return false;
+		}
 	}
-	else
+	else {
 		m_pOwner->LogMessage(MessageType::Debug_Info, _T("Trying to resume existing TLS session."));
+	}
 
 	return true;
 }
@@ -507,13 +556,21 @@ bool CTlsSocket::ResumedSession() const
 int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 {
 	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::Handshake()"));
-	wxASSERT(m_session);
+	if (!m_session) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Called CTlsSocket::Handshake without session"));
+		return FZ_REPLY_ERROR;
+	}
 
 	m_tlsState = TlsState::handshake;
 
 	fz::native_string hostname;
 
 	if (pPrimarySocket) {
+		if (!pPrimarySocket->m_session) {
+			m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Primary socket has no session"));
+			return FZ_REPLY_ERROR;
+		}
+
 		// Implicitly trust certificate of primary socket
 		unsigned int cert_list_size;
 		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
@@ -613,7 +670,7 @@ int CTlsSocket::Read(void *buffer, unsigned int len, int& error)
 	}
 
 	if (m_peekDataLen) {
-		unsigned int min = wxMin(len, m_peekDataLen);
+		auto min = std::min(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
 		if (min == m_peekDataLen) {
@@ -809,7 +866,7 @@ void CTlsSocket::Failure(int code, bool send_close, const wxString& function)
 int CTlsSocket::Peek(void *buffer, unsigned int len, int& error)
 {
 	if (m_peekData) {
-		int min = wxMin(len, m_peekDataLen);
+		auto min = std::min(len, m_peekDataLen);
 		memcpy(buffer, m_peekData, min);
 
 		error = 0;
@@ -924,20 +981,8 @@ static wxString bin2hex(const unsigned char* in, size_t size)
 }
 
 
-bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
+bool CTlsSocket::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& out)
 {
-	gnutls_x509_crt_t cert;
-	if (gnutls_x509_crt_init(&cert)) {
-		m_pOwner->LogMessage(MessageType::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
-		return false;
-	}
-
-	if (gnutls_x509_crt_import(cert, datum, GNUTLS_X509_FMT_DER)) {
-		m_pOwner->LogMessage(MessageType::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
-		gnutls_x509_crt_deinit(cert);
-		return false;
-	}
-
 	fz::datetime expirationTime(gnutls_x509_crt_get_expiration_time(cert), fz::datetime::seconds);
 	fz::datetime activationTime(gnutls_x509_crt_get_activation_time(cert), fz::datetime::seconds);
 
@@ -945,7 +990,7 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 	unsigned char buffer[40];
 	size_t size = sizeof(buffer);
 	int res = gnutls_x509_crt_get_serial(cert, buffer, &size);
-	if( res != 0 ) {
+	if (res != 0) {
 		size = 0;
 	}
 
@@ -956,16 +1001,18 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 	wxString pkAlgoName;
 	if (pkAlgo >= 0) {
 		const char* pAlgo = gnutls_pk_algorithm_get_name((gnutls_pk_algorithm_t)pkAlgo);
-		if (pAlgo)
+		if (pAlgo) {
 			pkAlgoName = wxString(pAlgo, wxConvUTF8);
+		}
 	}
 
 	int signAlgo = gnutls_x509_crt_get_signature_algorithm(cert);
 	wxString signAlgoName;
 	if (signAlgo >= 0) {
 		const char* pAlgo = gnutls_sign_algorithm_get_name((gnutls_sign_algorithm_t)signAlgo);
-		if (pAlgo)
+		if (pAlgo) {
 			signAlgoName = wxString(pAlgo, wxConvUTF8);
+		}
 	}
 
 	wxString subject, issuer;
@@ -975,20 +1022,20 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 	if (size) {
 		char* dn = new char[size + 1];
 		dn[size] = 0;
-		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size)))
-		{
+		if (!(res = gnutls_x509_crt_get_dn(cert, dn, &size))) {
 			dn[size] = 0;
 			subject = wxString(dn, wxConvUTF8);
 		}
-		else
+		else {
 			LogError(res, _T("gnutls_x509_crt_get_dn"));
+		}
 		delete [] dn;
 	}
-	else
+	else {
 		LogError(res, _T("gnutls_x509_crt_get_dn"));
+	}
 	if (subject.empty()) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
-		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
 
@@ -1003,15 +1050,16 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 			dn[size] = 0;
 			issuer = wxString(dn, wxConvUTF8);
 		}
-		else
+		else {
 			LogError(res, _T("gnutls_x509_crt_get_issuer_dn"));
+		}
 		delete [] dn;
 	}
-	else
+	else {
 		LogError(res, _T("gnutls_x509_crt_get_issuer_dn"));
+	}
 	if (issuer.empty() ) {
 		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
-		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
 
@@ -1030,10 +1078,13 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 		fingerprint_sha1 = bin2hex(digest, size);
 	}
 
-	gnutls_x509_crt_deinit(cert);
-
+	datum_holder der;
+	if (gnutls_x509_crt_export2(cert, GNUTLS_X509_FMT_DER, &der) != GNUTLS_E_SUCCESS) {
+		m_pOwner->LogMessage(MessageType::Error, _T("gnutls_x509_crt_get_issuer_dn"));
+		return false;
+	}
 	out = CCertificate(
-		datum->data, datum->size,
+		der.data, der.size,
 		activationTime, expirationTime,
 		serial,
 		pkAlgoName, pkBits,
@@ -1076,7 +1127,6 @@ std::vector<wxString> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
 			}
 		}
 	}
-
 	return ret;
 }
 
@@ -1140,6 +1190,62 @@ int CTlsSocket::GetAlgorithmWarnings()
 }
 
 
+bool CTlsSocket::GetSortedPeerCertificates(gnutls_x509_crt_t *& certs, unsigned int & certs_size)
+{
+	certs = 0;
+	certs_size = 0;
+
+	// First get unsorted list of peer certificates in DER
+	unsigned int cert_list_size;
+	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
+	if (!cert_list || !cert_list_size) {
+		m_pOwner->LogMessage(MessageType::Error, _("gnutls_certificate_get_peers returned no certificates"));
+		return false;
+	}
+
+	// Convert them all to PEM
+	gnutls_datum_t *pem_cert_list = new gnutls_datum_t[cert_list_size];
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		if (gnutls_pem_base64_encode2("CERTIFICATE", cert_list + i, pem_cert_list + i) != 0) {
+			for (unsigned int j = 0; j < i; ++j) {
+				gnutls_free(pem_cert_list[j].data);
+			}
+			delete [] pem_cert_list;
+			m_pOwner->LogMessage(MessageType::Error, _("gnutls_pem_base64_encode2 failed"));
+			return false;
+		}
+	}
+
+	// Concatenate them
+	gnutls_datum_t concated_certs{};
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		concated_certs.size += pem_cert_list[i].size;
+	}
+	concated_certs.data = new unsigned char[concated_certs.size];
+	concated_certs.size = 0;
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		memcpy(concated_certs.data + concated_certs.size, pem_cert_list[i].data, pem_cert_list[i].size);
+		concated_certs.size += pem_cert_list[i].size;
+	}
+
+	for (unsigned i = 0; i < cert_list_size; ++i) {
+		gnutls_free(pem_cert_list[i].data);
+	}
+	delete[] pem_cert_list;
+
+	// And now import the certificates
+	int res = gnutls_x509_crt_list_import2(&certs, &certs_size, &concated_certs, GNUTLS_X509_FMT_PEM, GNUTLS_X509_CRT_LIST_SORT);
+
+	delete[] concated_certs.data;
+
+	if (res != GNUTLS_E_SUCCESS) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not sort peer certificates"));
+		return false;
+	}
+
+	return true;
+}
+
 int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != TlsState::handshake) {
@@ -1155,8 +1261,41 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
+	cert_list_holder certs;
+	if (!GetSortedPeerCertificates(certs.certs, certs.certs_size)) {
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
+	gnutls_x509_crt_t root{};
+	clone_cert(certs.certs[certs.certs_size - 1], root);
+	if (!root) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not copy certificate"));
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
+	// Our trust-model is user-guided TOFU on the host's certificate.
+	//
+	// Here we validate the certificate chain sent by the server on the assumption
+	// that it's signed by a trusted CA.
+	//
+	// For now, add the highest certificate from the chain to trust list. Otherweise
+	// gnutls_certificate_verify_peers2 always stops with GNUTLS_CERT_SIGNER_NOT_FOUND
+	// at the highest certificate in the chain.
+	//
+	// Actual trust decision is done later by the user.
+	gnutls_x509_trust_list_t tlist;
+	gnutls_certificate_get_trust_list(m_certCredentials, &tlist);
+	if (gnutls_x509_trust_list_add_cas(tlist, &root, 1, 0) != 1) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not add trust"));
+		Failure(0, true);
+		return FZ_REPLY_ERROR;
+	}
+
 	unsigned int status = 0;
 	int const verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
+
 	if (verifyResult < 0) {
 		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_certificate_verify_peers2 returned %d with status %u"), verifyResult, status);
 		m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
@@ -1164,29 +1303,39 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
-	if (status & GNUTLS_CERT_REVOKED) {
-		m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
+	if (status != 0) {
+		if (status & GNUTLS_CERT_REVOKED) {
+			m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
+		}
+		else if (status & GNUTLS_CERT_SIGNATURE_FAILURE) {
+			m_pOwner->LogMessage(MessageType::Error, _("Certificate signature verification failed"));
+		}
+		else if (status & GNUTLS_CERT_INSECURE_ALGORITHM) {
+			m_pOwner->LogMessage(MessageType::Error, _("A certificate in the chain was signed using an insecure algorithm"));
+		}
+		else if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
+			m_pOwner->LogMessage(MessageType::Error, _("An issuer in the certificate chain is not a certificate authority"));
+		}
+		else if (status & GNUTLS_CERT_UNEXPECTED_OWNER) {
+			m_pOwner->LogMessage(MessageType::Error, _("The server's hostname does not match the certificate's hostname"));
+		}
+		else {
+			m_pOwner->LogMessage(MessageType::Error, _("Received certificate chain could not be verified. Verification status is %d."), status);
+		}
+
 		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
-	if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
-		m_pOwner->LogMessage(MessageType::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
-		Failure(0, true);
-		return FZ_REPLY_ERROR;
-	}
-
-	unsigned int cert_list_size;
-	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
-	if (!cert_list || !cert_list_size) {
-		m_pOwner->LogMessage(MessageType::Error, _("gnutls_certificate_get_peers returned no certificates"));
+	datum_holder cert_der{};
+	if (gnutls_x509_crt_export2(certs.certs[0], GNUTLS_X509_FMT_DER, &cert_der) != GNUTLS_E_SUCCESS) {
 		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (m_implicitTrustedCert.data) {
-		if (m_implicitTrustedCert.size != cert_list[0].size ||
-			memcmp(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size))
+		if (m_implicitTrustedCert.size != cert_der.size ||
+			memcmp(m_implicitTrustedCert.data, cert_der.data, cert_der.size))
 		{
 			m_pOwner->LogMessage(MessageType::Error, _("Primary connection and data connection certificates don't match."));
 			Failure(0, true);
@@ -1203,16 +1352,16 @@ int CTlsSocket::VerifyCertificate()
 	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
 
 	std::vector<CCertificate> certificates;
-	for (unsigned int i = 0; i < cert_list_size; ++i) {
+	certificates.reserve(certs.certs_size);
+	for (unsigned int i = 0; i < certs.certs_size; ++i) {
 		CCertificate cert;
-		if (ExtractCert(cert_list, cert))
+		if (ExtractCert(certs.certs[i], cert)) {
 			certificates.push_back(cert);
+		}
 		else {
 			Failure(0, true);
 			return FZ_REPLY_ERROR;
 		}
-
-		++cert_list;
 	}
 
 	if (CertificateIsBlacklisted(certificates)) {
@@ -1230,8 +1379,9 @@ int CTlsSocket::VerifyCertificate()
 		GetCipherName(),
 		GetMacName(),
 		algorithmWarnings,
-		certificates);
+		std::move(certificates));
 
+	// Finally, ask user to verify the certificate chain
 	m_pOwner->SendAsyncRequest(pNotification);
 
 	return FZ_REPLY_WOULDBLOCK;
