@@ -4,17 +4,20 @@
 #include "socket.h"
 
 struct external_ip_resolve_event_type;
-typedef CEvent<external_ip_resolve_event_type> CExternalIPResolveEvent;
+typedef fz::simple_event<external_ip_resolve_event_type> CExternalIPResolveEvent;
 
-class CExternalIPResolver final : public CEventHandler
+class CExternalIPResolver final : public fz::event_handler
 {
 public:
-	CExternalIPResolver(CEventHandler & handler);
+	CExternalIPResolver(fz::event_handler & handler);
 	virtual ~CExternalIPResolver();
 
+	CExternalIPResolver(CExternalIPResolver const&) = delete;
+	CExternalIPResolver& operator=(CExternalIPResolver const&) = delete;
+
 	bool Done() const { return m_done; }
 	bool Successful() const;
-	wxString GetIP() const;
+	std::string GetIP() const;
 
 	void GetExternalIP(const wxString& address, CSocket::address_family protocol, bool force = false);
 
@@ -25,15 +28,15 @@ protected:
 	wxString m_address;
 	CSocket::address_family m_protocol{};
 	unsigned long m_port{80};
-	CEventHandler * m_handler{};
+	fz::event_handler * m_handler{};
 
 	bool m_done{};
 
-	wxString m_data;
+	std::string m_data;
 
 	CSocket *m_pSocket{};
 
-	virtual void operator()(CEventBase const& ev);
+	virtual void operator()(fz::event_base const& ev);
 	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
 
 	void OnConnect(int error);
@@ -54,11 +57,11 @@ protected:
 
 	// HTTP data
 	void ResetHttpData(bool resetRedirectCount);
-	bool m_gotHeader;
-	int m_responseCode;
+	bool m_gotHeader{};
+	int m_responseCode{};
 	wxString m_responseString;
 	wxString m_location;
-	int m_redirectCount;
+	int m_redirectCount{};
 
 	enum transferEncodings
 	{
@@ -71,12 +74,12 @@ protected:
 
 	struct t_chunkData
 	{
-		bool getTrailer;
-		bool terminateChunk;
-		wxLongLong size;
+		bool getTrailer{};
+		bool terminateChunk{};
+		int64_t size{};
 	} m_chunkData;
 
-	bool m_finished;
+	bool m_finished{};
 };
 
 #endif //__EXTERNALIPRESOLVER_H__
