@@ -1,15 +1,16 @@
 #include <filezilla.h>
-#include "LocalTreeView.h"
-#include "queue.h"
+
+#include "dndobjects.h"
+#include "dragdropmanager.h"
+#include "drop_target_ex.h"
 #include "filezillaapp.h"
 #include "filter.h"
 #include "file_utils.h"
-#include <wx/dnd.h>
-#include "dndobjects.h"
+#include "graphics.h"
 #include "inputdialog.h"
-#include "dragdropmanager.h"
-#include "drop_target_ex.h"
+#include "LocalTreeView.h"
 #include "Options.h"
+#include "queue.h"
 
 #include <libfilezilla/local_filesys.hpp>
 
@@ -249,6 +250,7 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, C
 
 	state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	SetImageList(GetSystemImageList());
 
@@ -270,6 +272,8 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState& state, C
 #endif
 
 	SetDropTarget(new CLocalTreeViewDropTarget(this));
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*this);
 }
 
 CLocalTreeView::~CLocalTreeView()
@@ -400,7 +404,7 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 {
 	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives"));
 
-	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 
 	m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
 	if (m_pVolumeEnumeratorThread->Failed()) {
@@ -408,16 +412,15 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+	for (auto it = drives.begin(); it != drives.end(); ++it) {
 		wxString drive = *it;
 		if (drive.Right(1) == _T("\\"))
 			drive = drive.RemoveLast();
 
-		wxGetApp().AddStartupProfileRecord(wxString::Format(_T("CLocalTreeView::DisplayDrives adding drive %s"), drive.c_str()));
-		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, drive));
+		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, _T(""), false));
 		AppendItem(item, _T(""));
-		SortChildren(parent);
 	}
+	SortChildren(parent);
 
 	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives adding drives done"));
 
@@ -426,7 +429,7 @@ bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 
 #endif
 
-void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir /*=_T("")*/)
+void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, const wxString& knownSubdir)
 {
 	fz::local_filesys local_filesys;
 
@@ -851,8 +854,12 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 
 void CLocalTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
-	if (notification == STATECHANGE_LOCAL_DIR)
+	if (notification == STATECHANGE_LOCAL_DIR) {
 		SetDir(m_state.GetLocalDir().GetPath());
+	}
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		RefreshListing();
@@ -990,30 +997,33 @@ bool CLocalTreeView::CreateRoot()
 
 void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 {
-	if (!m_pVolumeEnumeratorThread)
+	if (!m_pVolumeEnumeratorThread) {
 		return;
+	}
 
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
 	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
 
-	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED)
-	{
+	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
 		delete m_pVolumeEnumeratorThread;
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter)
-	{
+	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
 		wxString drive = iter->volume;
 
 		wxTreeItemIdValue tmp;
 		wxTreeItemId item = GetFirstChild(m_drives, tmp);
-		while (item)
-		{
+		while (item) {
 			wxString name = GetItemText(item);
-			if (name == drive || name.Left(drive.Len() + 1) == drive + _T(" "))
-			{
-				SetItemText(item, drive + _T(" (") + iter->volumeName + _T(")"));
+			if (name == drive || name.Left(drive.Len() + 1) == drive + _T(" ")) {
+
+				if (!iter->volumeName.empty()) {
+					SetItemText(item, drive + _T(" (") + iter->volumeName + _T(")"));
+				}
+				if (iter->icon != -1) {
+					SetItemImage(item, iter->icon);
+				}
 				break;
 			}
 			item = GetNextChild(m_drives, tmp);
@@ -1380,7 +1390,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 		if (pData) {
 			bool wasLink;
 			int attributes;
-			enum fz::local_filesys::type type;
+			fz::local_filesys::type type;
 			fz::datetime date;
 			if (!path.empty() && path.Last() == fz::local_filesys::path_separator)
 				type = fz::local_filesys::get_file_info(fz::to_native(path + pData->m_known_subdir), wasLink, 0, &date, &attributes);
