@@ -468,7 +468,7 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, std::unique_ptr<
 	}
 }
 
-bool CQueueView::CanStartTransfer(const CServerItem& server_item, struct t_EngineData *&pEngineData)
+bool CQueueView::CanStartTransfer(const CServerItem& server_item, t_EngineData *&pEngineData)
 {
 	const CServer &server = server_item.GetServer();
 	const int max_count = server.MaximumMultipleConnections();
@@ -1356,8 +1356,7 @@ void CQueueView::CheckQueueState()
 
 		CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
 
-		if (!m_quit)
-			ActionAfter();
+		ActionAfter();
 	}
 
 	if (m_quit)
@@ -1486,8 +1485,9 @@ void CQueueView::LoadQueue()
 		error = true;
 	else {
 		CServer server;
-		int64_t id;
-		for (id = m_queue_storage.GetServer(server, true); id > 0; id = m_queue_storage.GetServer(server, false)) {
+		int64_t const first_id = m_queue_storage.GetServer(server, true);
+		auto id = first_id;
+		for (; id > 0; id = m_queue_storage.GetServer(server, false)) {
 			m_insertionStart = -1;
 			m_insertionCount = 0;
 			CServerItem *pServerItem = CreateServerItem(server);
@@ -1511,21 +1511,32 @@ void CQueueView::LoadQueue()
 		if (id < 0)
 			error = true;
 
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 2)
-			if (!m_queue_storage.Clear())
+		if (error || first_id > 0) {
+			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 2) {
+				if (!m_queue_storage.Clear()) {
+					error = true;
+				}
+			}
+
+			if (!m_queue_storage.EndTransaction()) {
 				error = true;
+			}
 
-		if (!m_queue_storage.EndTransaction())
-			error = true;
-
-		if (!m_queue_storage.Vacuum())
-			error = true;
+			if (!m_queue_storage.Vacuum()) {
+				error = true;
+			}
+		}
+		else {
+			// Queue was already empty. No need to commit
+			if (!m_queue_storage.EndTransaction(true)) {
+				error = true;
+			}
+		}
 	}
 
 	m_insertionStart = -1;
 	m_insertionCount = 0;
 	CommitChanges();
-
 	if (error) {
 		wxString file = CQueueStorage::GetDatabaseFilename();
 		wxString msg = wxString::Format(_("An error occurred loading the transfer queue from \"%s\".\nSome queue items might not have been restored."), file);
@@ -2448,6 +2459,10 @@ bool CQueueView::IsActionAfter(ActionAfterState::type state)
 
 void CQueueView::ActionAfter(bool warned /*=false*/)
 {
+	if (m_quit) {
+		return;
+	}
+
 	// Need to check all contexts whether there's a recursive
 	// download operation still in progress
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
@@ -2860,17 +2875,15 @@ wxFileOffset CQueueView::GetCurrentSpeed(bool countDownload, bool countUpload)
 {
 	wxFileOffset totalSpeed = 0;
 
-	for (auto iter = m_statusLineList.begin(); iter != m_statusLineList.end(); ++iter)
-	{
-		CStatusLineCtrl *pCtrl = *iter;
+	for (auto pCtrl : m_statusLineList) {
 		const CFileItem *pItem = pCtrl->GetItem();
 		bool isDownload = pItem->Download();
 
-		if ((isDownload && countDownload) || (!isDownload && countUpload))
-		{
-			wxFileOffset speed = pCtrl->GetCurrentSpeed();
-			if (speed > 0)
+		if ((isDownload && countDownload) || (!isDownload && countUpload)) {
+			wxFileOffset speed = pCtrl->GetMomentarySpeed();
+			if (speed > 0) {
 				totalSpeed += speed;
+			}
 		}
 	}
 
