@@ -1,18 +1,20 @@
 #include <filezilla.h>
-#include "RemoteTreeView.h"
-#include "commandqueue.h"
-#include <wx/dnd.h>
-#include "dndobjects.h"
+
 #include "chmoddialog.h"
-#include "remote_recursive_operation.h"
-#include "inputdialog.h"
+#include "commandqueue.h"
+#include "dndobjects.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
-#include <wx/clipbrd.h>
+#include "graphics.h"
+#include "inputdialog.h"
+#include "Options.h"
 #include "queue.h"
 #include "QueueView.h"
+#include "RemoteTreeView.h"
+#include "remote_recursive_operation.h"
 #include "themeprovider.h"
-#include "Options.h"
+
+#include <wx/clipbrd.h>
 
 #include <algorithm>
 
@@ -251,6 +253,7 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state,
 
 	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	CreateImageList();
 
@@ -265,6 +268,8 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState& state,
 	SetDropTarget(new CRemoteTreeViewDropTarget(this));
 
 	Enable(false);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*this);
 }
 
 CRemoteTreeView::~CRemoteTreeView()
@@ -273,12 +278,17 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void* v)
+void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, const wxString&, const void* data2)
 {
-	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(m_state.GetRemoteDir(), v ? *reinterpret_cast<bool const*>(v) : false);
-	else if (notification == STATECHANGE_APPLYFILTER)
+	if (notification == STATECHANGE_REMOTE_DIR) {
+		SetDirectoryListing(m_state.GetRemoteDir(), data2 ? *reinterpret_cast<bool const*>(data2) : false);
+	}
+	else if (notification == STATECHANGE_APPLYFILTER) {
 		ApplyFilters(false);
+	}
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 }
 
 void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified)
@@ -1298,7 +1308,7 @@ struct _parents
 
 void CRemoteTreeView::ApplyFilters(bool resort)
 {
-	std::list<struct _parents> parents;
+	std::list<_parents> parents;
 
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemIdValue cookie;
@@ -1307,7 +1317,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 		if (path.empty())
 			continue;
 
-		struct _parents dir;
+		_parents dir;
 		dir.item = child;
 		dir.path = path;
 		parents.push_back(dir);
@@ -1315,7 +1325,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 
 	CFilterManager filter;
 	while (!parents.empty()) {
-		struct _parents parent = parents.back();
+		_parents parent = parents.back();
 		parents.pop_back();
 
 		if (resort) {
@@ -1341,7 +1351,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 					}
 				}
 
-				struct _parents dir;
+				_parents dir;
 				dir.item = child;
 				dir.path = path;
 				parents.push_back(dir);
@@ -1355,7 +1365,7 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 			if (path.empty())
 				continue;
 
-			struct _parents dir;
+			_parents dir;
 			dir.item = child;
 			dir.path = path;
 			parents.push_back(dir);
