@@ -3,6 +3,7 @@
 #include "aboutdialog.h"
 #include "buildinfo.h"
 #include "xrc_helper.h"
+#include "Options.h"
 
 #include <misc.h>
 
@@ -20,7 +21,7 @@ bool CAboutDialog::Create(wxWindow* parent)
 		return false;
 
 	xrc_call(*this, "ID_URL", &wxHyperlinkCtrl::SetLabel, _T("https://filezilla-project.org/"));
-	xrc_call(*this, "ID_COPYRIGHT", &wxStaticText::SetLabel, _T("Copyright (C) 2004-2015  Tim Kosse"));
+	xrc_call(*this, "ID_COPYRIGHT", &wxStaticText::SetLabel, _T("Copyright (C) 2004-2016  Tim Kosse"));
 
 	wxString version = CBuildInfo::GetVersion();
 	if (CBuildInfo::GetBuildType() == _T("nightly"))
@@ -58,13 +59,13 @@ bool CAboutDialog::Create(wxWindow* parent)
 		xrc_call(*this, "ID_CFLAGS_DESC", &wxStaticText::Hide);
 	}
 	else {
-		WrapText(this, compilerFlags, 200);
+		WrapText(this, compilerFlags, 250);
 		xrc_call(*this, "ID_CFLAGS", &wxStaticText::SetLabel, compilerFlags);
 	}
 
-	xrc_call(*this, "ID_VER_WX", &wxStaticText::SetLabel, GetDependencyVersion(dependency::wxwidgets));
-	xrc_call(*this, "ID_VER_GNUTLS", &wxStaticText::SetLabel, GetDependencyVersion(dependency::gnutls));
-	xrc_call(*this, "ID_VER_SQLITE", &wxStaticText::SetLabel, GetDependencyVersion(dependency::sqlite));
+	xrc_call(*this, "ID_VER_WX", &wxStaticText::SetLabel, GetDependencyVersion(lib_dependency::wxwidgets));
+	xrc_call(*this, "ID_VER_GNUTLS", &wxStaticText::SetLabel, GetDependencyVersion(lib_dependency::gnutls));
+	xrc_call(*this, "ID_VER_SQLITE", &wxStaticText::SetLabel, GetDependencyVersion(lib_dependency::sqlite));
 
 	wxString const os = wxGetOsDescription();
 	if (os.empty()) {
@@ -100,6 +101,18 @@ bool CAboutDialog::Create(wxWindow* parent)
 	xrc_call(*this, "ID_SYSTEM_PLATFORM_DESC", &wxStaticText::Hide);
 #endif
 
+	wxString cpuCaps = CBuildInfo::GetCPUCaps(' ');
+	if (!cpuCaps.empty()) {
+		WrapText(this, cpuCaps, 250);
+		xrc_call(*this, "ID_SYSTEM_CPU", &wxStaticText::SetLabel, cpuCaps);
+	}
+	else {
+		xrc_call(*this, "ID_SYSTEM_CPU_DESC", &wxStaticText::Hide);
+		xrc_call(*this, "ID_SYSTEM_CPU", &wxStaticText::Hide);
+	}
+
+	xrc_call(*this, "ID_SYSTEM_SETTINGS_DIR", &wxStaticText::SetLabel, COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR));
+
 	GetSizer()->Fit(this);
 	GetSizer()->SetSizeHints(this);
 
@@ -140,10 +153,10 @@ void CAboutDialog::OnCopy(wxCommandEvent&)
 		text += _T("  Compiler flags: ") + compilerFlags + _T("\n");
 
 	text += _T("\nLinked against:\n");
-	for (int i = 0; i < dependency::count; ++i) {
+	for (int i = 0; i < static_cast<int>(lib_dependency::count); ++i) {
 		text += wxString::Format(_T("  % -15s %s\n"),
-			GetDependencyName(dependency::type(i)) + _T(":"),
-			GetDependencyVersion(dependency::type(i)));
+			GetDependencyName(lib_dependency(i)) + _T(":"),
+			GetDependencyVersion(lib_dependency(i)));
 	}
 
 	text += _T("\nOperating system:\n");
@@ -168,6 +181,16 @@ void CAboutDialog::OnCopy(wxCommandEvent&)
 		text += _T("  Platform:       64-bit system\n");
 	else
 		text += _T("  Platform:       32-bit system\n");
+#endif
+
+	wxString cpuCaps = CBuildInfo::GetCPUCaps(' ');
+	if (!cpuCaps.empty()) {
+		text += _T("  CPU features:   ") + cpuCaps + _T("\n");
+	}
+
+	text += _T("  Settings dir:   ") + COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + _T("\n");
+
+#ifdef __WXMSW__
 	text.Replace(_T("\n"), _T("\r\n"));
 #endif
 
