@@ -383,6 +383,14 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 	pSizer->Add(m_pNotebook_Bookmark, 1, wxGROW);
 	pSizer->SetItemMinSize(1, pSizer->GetItem((size_t)0)->GetMinSize().GetWidth(), -1);
 
+	// Set min size of tree to actual size of tree.
+	// Otherwise some platforms automatically calculate a min size fitting all items,
+	// resulting in a huge dialog if there are many sites.
+	wxSize const treeSize = pTree->GetSize();
+	if (treeSize.IsFullySpecified()) {
+		pTree->SetMinSize(treeSize);
+	}
+
 	if (!Load()) {
 		return false;
 	}
@@ -1442,7 +1450,7 @@ bool CSiteManagerDialog::UpdateServer(Site &server, const wxString &name)
 	return true;
 }
 
-bool CSiteManagerDialog::GetServer(Site& data)
+bool CSiteManagerDialog::GetServer(Site& data, Bookmark& bookmark)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree) {
@@ -1463,17 +1471,20 @@ bool CSiteManagerDialog::GetServer(Site& data)
 		CSiteManagerItemData* pSiteData = static_cast<CSiteManagerItemData*>(pTree->GetItemData(item));
 
 		data = *pSiteData->m_site;
+		bookmark = data.m_default_bookmark;
+
 		if (!pData->m_bookmark->m_localDir.empty())
-			data.m_default_bookmark.m_localDir = pData->m_bookmark->m_localDir;
+			bookmark.m_localDir = pData->m_bookmark->m_localDir;
 		if (!pData->m_bookmark->m_remoteDir.empty())
-			data.m_default_bookmark.m_remoteDir = pData->m_bookmark->m_remoteDir;
-		if (data.m_default_bookmark.m_localDir.empty() || data.m_default_bookmark.m_remoteDir.empty())
-			data.m_default_bookmark.m_sync = false;
+			bookmark.m_remoteDir = pData->m_bookmark->m_remoteDir;
+		if (bookmark.m_localDir.empty() || bookmark.m_remoteDir.empty())
+			bookmark.m_sync = false;
 		else
-			data.m_default_bookmark.m_sync = pData->m_bookmark->m_sync;
+			bookmark.m_sync = pData->m_bookmark->m_sync;
 	}
 	else {
 		data = *pData->m_site;
+		bookmark = data.m_default_bookmark;
 	}
 
 	data.m_path = GetSitePath(item);
