@@ -1,6 +1,7 @@
 #include <filezilla.h>
 #include "viewheader.h"
 #include "commandqueue.h"
+#include "graphics.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/uxtheme.h>
@@ -83,6 +84,8 @@ CViewHeader::CViewHeader(wxWindow* pParent, const wxString& label)
 	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 	m_pComboBox->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 	m_pLabel->SetBackgroundStyle(wxBG_STYLE_SYSTEM);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*m_pComboBox);
 }
 
 void CViewHeader::OnSize(wxSizeEvent&)
@@ -114,8 +117,7 @@ void CViewHeader::OnSize(wxSizeEvent&)
 void CViewHeader::OnComboPaint(wxPaintEvent& event)
 {
 	// We do a small trick to let the control handle the event before we can paint
-	if (m_alreadyInPaint)
-	{
+	if (m_alreadyInPaint) {
 		event.Skip();
 		return;
 	}
@@ -132,8 +134,7 @@ void CViewHeader::OnComboPaint(wxPaintEvent& event)
 
 	int thumbWidth = ::GetSystemMetrics(SM_CXHTHUMB);
 
-	if (m_bLeftMousePressed)
-	{
+	if (m_bLeftMousePressed) {
 		if (!SendMessage((HWND)box->GetHandle(), CB_GETDROPPEDSTATE, 0, 0))
 			m_bLeftMousePressed = false;
 	}
@@ -334,6 +335,7 @@ CLocalViewHeader::CLocalViewHeader(wxWindow* pParent, CState& state)
 	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(state)
 {
 	state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 }
 
 void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
@@ -499,15 +501,17 @@ void CLocalViewHeader::OnTextEnter(wxCommandEvent&)
 
 void CLocalViewHeader::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
-	wxASSERT(notification == STATECHANGE_LOCAL_DIR);
-	(void)notification;
-
+	if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
+	else if (notification == STATECHANGE_LOCAL_DIR) {
 #ifdef __WXGTK__
-	m_autoCompletionText = _T("");
+		m_autoCompletionText = _T("");
 #endif
 
-	wxString dir = m_state.GetLocalDir().GetPath();
-	AddRecentDirectory(dir);
+		wxString dir = m_state.GetLocalDir().GetPath();
+		AddRecentDirectory(dir);
+	}
 }
 
 BEGIN_EVENT_TABLE(CRemoteViewHeader, CViewHeader)
@@ -519,32 +523,35 @@ CRemoteViewHeader::CRemoteViewHeader(wxWindow* pParent, CState& state)
 	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(state)
 {
 	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 	Disable();
 }
 
 void CRemoteViewHeader::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
-	wxASSERT(notification == STATECHANGE_REMOTE_DIR);
-	(void)notification;
-
-	m_path = m_state.GetRemotePath();
-	if (m_path.empty()) {
-		m_pComboBox->SetValue(_T(""));
-		Disable();
+	if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
 	}
-	else {
-		const CServer* const pServer = m_state.GetServer();
-		if (pServer && *pServer != m_lastServer) {
-			m_pComboBox->Clear();
-			m_recentDirectories.clear();
-			m_sortedRecentDirectories.clear();
-			m_lastServer = *pServer;
+	else if (notification == STATECHANGE_REMOTE_DIR) {
+		m_path = m_state.GetRemotePath();
+		if (m_path.empty()) {
+			m_pComboBox->SetValue(_T(""));
+			Disable();
 		}
-		Enable();
+		else {
+			const CServer* const pServer = m_state.GetServer();
+			if (pServer && *pServer != m_lastServer) {
+				m_pComboBox->Clear();
+				m_recentDirectories.clear();
+				m_sortedRecentDirectories.clear();
+				m_lastServer = *pServer;
+			}
+			Enable();
 #ifdef __WXGTK__
-		GetParent()->m_dirtyTabOrder = true;
+			GetParent()->m_dirtyTabOrder = true;
 #endif
-		AddRecentDirectory(m_path.GetPath());
+			AddRecentDirectory(m_path.GetPath());
+		}
 	}
 }
 
