@@ -1240,7 +1240,9 @@ int sftp_cmd_ls(struct sftp_command *cmd)
 	    if (ournames[i]->attrs.flags & SSH_FILEXFER_ATTR_ACMODTIME) {
 		mtime = ournames[i]->attrs.mtime;
 	    }
-	    fzprintf(sftpListentry, "%lu %s", mtime, ournames[i]->longname);
+	    fzprintf_raw_untrusted(sftpListentry, ournames[i]->longname);
+	    fzprintf_raw_untrusted(sftpUnknown, "%lu", mtime);
+	    fzprintf_raw_untrusted(sftpUnknown, "%s", ournames[i]->filename);
 	    fxp_free_name(ournames[i]);
 	}
 	sfree(ournames);
