@@ -131,7 +131,6 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	"If you do not trust this host, press Return to abandon the\n"
 	"connection.\n"
 	"Store key in cache? (y/n) ";
-*/
     static const char wrongmsg_batch[] =
 	"WARNING - POTENTIAL SECURITY BREACH!\n"
 	"The server's host key does not match the one PuTTY has\n"
@@ -141,7 +140,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	"The new %s key fingerprint is:\n"
 	"%s\n"
 	"Connection abandoned.\n";
-/*    static const char wrongmsg[] =
+    static const char wrongmsg[] =
 	"WARNING - POTENTIAL SECURITY BREACH!\n"
 	"The server's host key does not match the one PuTTY has\n"
 	"cached. This means that either the server administrator\n"
@@ -172,20 +171,7 @@ int verify_ssh_host_key(void *frontend, char *host, int port,
 	return 1;
 
 //FZ premsg(&cf);
-    if (ret == 2) {		       /* key was different */
-	if (console_batch_mode) {
-	    fprintf(stderr, wrongmsg_batch, keytype, fingerprint);
-	    return 0;
-	}
-	fzprintf_raw(sftpRequest, "%d%s\n%d\n%s\n", (int)sftpReqHostkeyChanged, host, port, fingerprint);
-    }
-    if (ret == 1) {		       /* key was absent */
-	if (console_batch_mode) {
-	    fprintf(stderr, absentmsg_batch, keytype, fingerprint);
-	    return 0;
-	}
-	fzprintf_raw(sftpRequest, "%d%s\n%d\n%s\n", (int)sftpReqHostkey, host, port, fingerprint);
-    }
+    fzprintf_raw((ret == 1) ? sftpAskHostkey : sftpAskHostkeyChanged, "%s\n%d\n%s\n", host, port, fingerprint);
 
     {
 	struct termios oldmode, newmode;
@@ -254,7 +240,7 @@ int askhk(void *frontend, const char *algname, const char *betteralgs,
 	return 0;
     }
 
-    fzprintf_raw(sftpRequest, "%d%s\n%s\n", (int)sftpReqHostkeyBetteralg, algname, betteralgs);
+    fzprintf_raw(sftpAskHostkeyBetteralg, "%d%s\n%s\n", algname, betteralgs);
 
     {
 	struct termios oldmode, newmode;
@@ -439,13 +425,13 @@ int console_get_userpass_input(prompts_t *p, const unsigned char *in,
      */
     /* We only print the `name' caption if we have to... */
     if (p->name)
-	fzprintf_raw_untrusted(sftpRequestPreamble, p->name);
+	fzprintf_raw_untrusted(sftpRequestPreamble, "%s", p->name);
     else
 	fzprintf_raw_untrusted(sftpRequestPreamble, "");
 
     /* ...but we always print any `instruction'. */
     if (p->instruction)
-	fzprintf_raw_untrusted(sftpRequestInstruction, p->instruction);
+	fzprintf_raw_untrusted(sftpRequestInstruction, "%s", p->instruction);
     else
 	fzprintf_raw_untrusted(sftpRequestInstruction, "");
 
@@ -464,7 +450,7 @@ int console_get_userpass_input(prompts_t *p, const unsigned char *in,
 //	    newmode.c_lflag |= ECHO;
 	tcsetattr(infd, TCSANOW, &newmode);
 
-	fzprintf_raw_untrusted(sftpRequest, "%d%s\n", (int)sftpReqPassword, pr->prompt);
+	fzprintf_raw_untrusted(sftpAskPassword, "%s", pr->prompt);
 
         len = 0;
         while (1) {
